package com.fundi.framework.common.navigation;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;

import com.fundi.framework.common.instance.AppInstance;
import com.fundi.framework.common.nl1.Messages;
import com.fundi.framework.common.util.Collator;

public class NavigationItem implements Comparable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$

	private SerializableNavigationItem navItem;
	private Object data = null;  //Typically the sourcing object, e.g. an ICON system, an IMSplex.
	private ArrayList<Object> alternateData = new ArrayList<Object>();  //Holds alternative data objects from other than the sourcing plug-in.
	private ArrayList<NavigationItem> subItems = new ArrayList<NavigationItem>();  //Holds viewable (after filtering) sub items.
	private ArrayList<NavigationItem> unfilteredSubItems = new ArrayList<NavigationItem>();  //Holds all sub items.
	private String imagePath = ""; //$NON-NLS-1$
	private String statusImagePath = ""; //$NON-NLS-1$
	
	private NavigationProvider provider = null;
	
	private AppInstance instance = null;
	
	public NavigationItem() {
		
		super();
	}

	public NavigationItem(AppInstance instance) {
		
		super();
		
		this.instance = instance;
		
		navItem = new SerializableNavigationItem();
	}
	
	public NavigationItem(SerializableNavigationItem navItem) {

		super();
		
		this.navItem = navItem;
	}
	
	public NavigationItem(String displayCaption, Object data, NavigationItem parentItem) {

		super();
		
		this.instance = instance;
		
		navItem = new SerializableNavigationItem();
		navItem.setDisplayCaption(displayCaption);
		if (data != null) {
			navItem.setDataClassName(data.getClass().getName());
			this.data = data;
		}
		navItem.setFullTitle(getFullTitle());  //Must be done after data is set.
		
		if (parentItem != null) {
			parentItem.addSubItem(this);
		}
	}
	
	public void addSubItem(NavigationItem item) {
		
		/*
		 *This method doesn't simply call SerializableNavigationItem.addSubItem() because sub-items are typically only used in the navigation view
		 *to display the tree, so SerializableNavigationItem doesn't support all the properties of sub-items (or items in general).
		 *
		 *That is, sub-items really only make sense in the context of maintaining, saving and restoring the navigation tree.
		 */
		
		Iterator<NavigationItem> itSub;
		boolean found = false;
		
		for (itSub = subItems.iterator(); itSub.hasNext();) {
			if (itSub.next().equals(item)) {
				found = true;
				break;
			}
		}
		
		if (!found) {
			subItems.add(item);
			navItem.setGroup(true);
			navItem.addSubItem(item.serialize());
		}
	}
	
	public void setSubItems(ArrayList<NavigationItem> items) {
		
		int idx;
		
		removeAllSubItems();
		for (idx = 0; idx < items.size(); idx++) {
			addSubItem(items.get(idx));
		}
	}
	
	public void addUnfilteredSubItem(NavigationItem item) {
		
		Iterator<NavigationItem> itSub;
		boolean found = false;
		
		for (itSub = unfilteredSubItems.iterator(); itSub.hasNext();) {
			if (itSub.next().equals(item)) {
				found = true;
				break;
			}
		}
		
		if (!found) {
			unfilteredSubItems.add(item);
		}
	}
	
	public ArrayList<NavigationItem> getSubItems() {
		
		return getSubItems(false);
	}
	
	public ArrayList<NavigationItem> getSubItems(boolean ignoreTypes) {
		
		ArrayList<NavigationItem> subs = new ArrayList<NavigationItem>();
		Iterator<NavigationItem> itSub;
		NavigationItem subItem;
		
		for (itSub = subItems.iterator(); itSub.hasNext();) {
			subItem = itSub.next();
			if (ignoreTypes) {
				if (String.class.isInstance(subItem.getData())) {
					if (subItem.getDisplayCaption().equals((String) subItem.getData())) {
						//It's a type, so get its sub-items.
						subs.addAll(subItem.getSubItems());
					}
					else {
						subs.add(subItem);
					}
				}
				else {
					subs.add(subItem);
				}
			}
			else {
				subs.add(subItem);
			}
		}
		
		return subs;
	}

	public ArrayList<NavigationItem> getUnfilteredSubItems() {
		
		return getUnfilteredSubItems(false);
	}
	
	public ArrayList<NavigationItem> getUnfilteredSubItems(boolean ignoreTypes) {
		
		ArrayList<NavigationItem> subs = new ArrayList<NavigationItem>();
		Iterator<NavigationItem> itSub;
		NavigationItem subItem;
		
		for (itSub = unfilteredSubItems.iterator(); itSub.hasNext();) {
			subItem = itSub.next();
			if (ignoreTypes) {
				if (String.class.isInstance(subItem.getData())) {
					if (subItem.getDisplayCaption().equals((String) subItem.getData())) {
						//It's a type, so get its sub-items.
						subs.addAll(subItem.getUnfilteredSubItems());
					}
					else {
						subs.add(subItem);
					}
				}
				else {
					subs.add(subItem);
				}
			}
			else {
				subs.add(subItem);
			}
		}
		
		return subs;
	}
	
	public ArrayList<NavigationItem> getSubItemsUnaltered() {
		
		return subItems;
	}

	public void removeAllSubItems() {
		
		navItem.removeAllSubItems();
		subItems.clear();
	}
	
	public void removeSubItem(NavigationItem subItem) {

		navItem.removeSubItem(subItem.getSerialized());
		subItems.remove(subItem);
	}

	public SerializableNavigationItem getSerialized() {
		
		return navItem;
	}
	
	public void setGroup(boolean group) {
		
		navItem.setGroup(group);
	}

	public boolean isExpanded() {
		
		return navItem.isExpanded();
	}

	public void setExpanded(boolean expanded) {
		
		navItem.setExpanded(expanded);
	}

	public boolean isOpened() {
		
		return navItem.isOpened();
	}

	public void setOpened(boolean opened) {
		
		navItem.setOpened(opened);
	}

	public String getDisplayCaption() {
		
		return navItem.getDisplayCaption();
	}

	public void setDisplayCaption(String displayCaption) {
		
		navItem.setDisplayCaption(displayCaption);
	}

	public String getDataClassName() {
		
		return navItem.getDataClassName();
	}

	public void setDataClassName(String dataClassName) {
		
		navItem.setDataClassName(dataClassName);
	}

	public String getImagePath() {
		
		return imagePath;
	}

	public void setImagePath(String imagePath) {
		
		this.imagePath = imagePath;
	}

	public String getStatusImagePath() {
		
		return statusImagePath;
	}

	public void setStatusImagePath(String statusImagePath) {
		
		this.statusImagePath = statusImagePath;
	}
	
	public SerializableNavigationItem serialize() {
		
		return navItem;
	}

	public Object getData() {
		
		return data;
	}

	public void setData(Object data) {
		
		this.data = data;
	}
	
	public Object getData(Class dataClass) {

		int idx;
		Object alt;
		
		if (data != null) {
			if (data.getClass() == dataClass) {
				return data;
			}
		}
		
		for (idx = 0; idx < alternateData.size(); idx++) {
			alt = alternateData.get(idx);
			if (alt.getClass() == dataClass) {
				return alt;
			}
		}
		
		return null;
	}
	
	public Object getAlternateData(Class altClass) {
		
		int idx;
		Object alt;
		
		for (idx = 0; idx < alternateData.size(); idx++) {
			alt = alternateData.get(idx);
			if (alt.getClass() == altClass) {
				return alt;
			}
		}
		
		return null;
	}
	
	public ArrayList<Object> getAlternateData() {
		
		return alternateData;
	}
	
	public void setAlternateData(Object altData) {
		
		alternateData.add(altData);
	}

	public NavigationProvider getProvider() {
		
		return provider;
	}

	public void setProvider(NavigationProvider provider) {
		
		this.provider = provider;
	}
	
	public void setFullTitle() {
		
		navItem.setFullTitle(getFullTitle());
	}
	
	public String getFullTitle() {
		
		String title = ""; //$NON-NLS-1$
		Method method = null;
		
		if (data != null) {
			try {
				method = data.getClass().getMethod("getFullTitle", new Class[] {}); //$NON-NLS-1$
			} 			
			catch (SecurityException e) {
				instance.addConsoleMessage(e);
			} 	
			catch (NoSuchMethodException e) {}
			if (method != null) {
				try {
					title = (String) method.invoke(data, new Object[] {});
				} 			
				catch (Throwable e) {			
					instance.addConsoleMessage(e);
				}
			}
		}
		
		if (title.length() > 0) {
			return title;
		}
		else {
			return getDisplayCaption();
		}
	}
	
	public String getFullTitleWithoutInitialType() {
		
		int startPos;
		int endPos;
		String title = getFullTitle();
		
		startPos = title.indexOf(" "); //$NON-NLS-1$
		if (startPos > -1 && startPos < (title.length() - 1)) {
			if (title.substring(startPos + 1, startPos + 2).equals("[")) { //$NON-NLS-1$
				endPos = title.indexOf("]"); //$NON-NLS-1$
				if (endPos > -1 && endPos < (title.length() - 1)) {
					title = title.substring(0, startPos) + title.substring(endPos + 1);
				}
			}
		}
		
		return title;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((getDataClassName() == null) ? 0 : getDataClassName().hashCode());
		result = prime * result
				+ ((getDisplayCaption() == null) ? 0 : getDisplayCaption().hashCode());
		result = prime * result
				+ ((getFullTitle() == null) ? 0 : getFullTitle().hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		NavigationItem other = (NavigationItem) obj;
		if (getDataClassName() == null) {
			if (other.getDataClassName() != null)
				return false;
		} else if (!getDataClassName().equals(other.getDataClassName()))
			return false;
		if (getDisplayCaption() == null) {
			if (other.getDisplayCaption() != null)
				return false;
		} else if (!getDisplayCaption().equals(other.getDisplayCaption()))
			return false;
		if (getFullTitle() == null) {
			if (other.getFullTitle() != null)
				return false;
		} else if (!getFullTitle().equals(other.getFullTitle()))
			return false;
		return true;
	}

	@Override
	public int compareTo(Object arg0) {
		
		return (new Collator()).compare(getFullTitle(), ((NavigationItem) arg0).getFullTitle());
	}
	
	public boolean isGroup() {
		return navItem.isGroup();
	}
}
